package vismon.data;

import vismon.gui.OptionsPanel;
import vismon.gui.Vismon;
import vismon.util.Computations;
import vismon.util.Constants;
import vismon.util.Util;

import java.awt.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


public class Session {

    private static String                   dataSetName;

    private static List<MngOption>          mngOptions;
    private static int                      mngOptionNum;   // = 2

    private static List<Indicator>          indicators;
    private static int                      indicatorNum;
    private static int                      indicatorRowNum;
    private static int                      indicatorColNum;

    private static List<DataItem>           dataItems;
    private static int                      simulationNum;

    private static List<VisualRefLine>      visualRefLines;

    private static List<Scenario>           scenarios;
    private static Scenario                 selectedScenario;
    private static Scenario                 crosshairScenario;

    private static int                      errorBarPercentage;



    static {
        mngOptions = new ArrayList<MngOption>();
        indicators = new ArrayList<Indicator>();
        dataItems = new ArrayList<DataItem>();
        visualRefLines = new ArrayList<VisualRefLine>();
        scenarios = new ArrayList<Scenario>();
        errorBarPercentage = 95;
    }

    public static String getDataSetName() {
        return dataSetName;
    }

    public static void setDataSetName(String dataSetName) {
        Session.dataSetName = dataSetName;
    }

    public static int getIndicatorNum() {
        return indicatorNum;
    }

    public static void setIndicatorNum(int indicatorNum) {
        Session.indicatorNum = indicatorNum;
    }

    public static List<Indicator> getIndicators() {
        return indicators;
    }

    public static Indicator getIndicator(int rank) {
        return indicators.get(rank);
    }

    public static void addIndicator(Indicator indicator) {
        Session.indicators.add(indicator);
    }

    public static int getMngOptionNum() {
        return mngOptionNum;
    }

    public static void setMngOptionNum(int mngOptionNum) {
        Session.mngOptionNum = mngOptionNum;
    }

    public static List<MngOption> getMngOptions() {
        return mngOptions;
    }

    public static MngOption getMngOption(int rank) {
        return mngOptions.get(rank);
    }

    public static void addMngOption(MngOption mngOption) {
        Session.mngOptions.add(mngOption);
    }

    public static List<VisualRefLine> getVisualRefLines() {
        return visualRefLines;
    }

    public static void addVisualRefLine(VisualRefLine visualRefLine) {
        visualRefLines.add(visualRefLine);
    }

    public static void removeVisualRefLine(int rank) {
        visualRefLines.remove(rank);
    }

    public static List<Scenario> getScenarios() {
        return scenarios;
    }

    public static Scenario getScenario(int rank) {
        return scenarios.get(rank);
    }

    public static void addScenario(Scenario scenario) {
        Session.scenarios.add(scenario);
        Constants.usedScenarioColor(scenario.getColor());

        Vismon.getVismonInstance().getScenarioPanel().addRow(scenario);
        Vismon.getVismonInstance().updatePlots();
    }

    public static void removeScenario(int rank) {
        Constants.freeScenarioColor(scenarios.get(rank).getColor());
        scenarios.remove(rank);

        Vismon.getVismonInstance().getScenarioPanel().removeRow(rank);
        Vismon.getVismonInstance().updatePlots();
    }

    public static void removeAllScenarios() {
        while (scenarios.size() != 0)
            removeScenario(scenarios.size() - 1);
    }

    public static Scenario createScenario(double xMngValue, double yMngValue) {
        MngAction mngAction = new MngAction();
        double xMngValueRounded = Math.round(xMngValue / getMngOption(0).getRoundFactor()) * getMngOption(0).getRoundFactor();
        double yMngValueRounded = Math.round(yMngValue / getMngOption(1).getRoundFactor()) * getMngOption(1).getRoundFactor();
        mngAction.addMngValue(getMngOption(0), xMngValueRounded);
        mngAction.addMngValue(getMngOption(1), yMngValueRounded);

        Scenario scenario = new Scenario(mngAction);
        Scenario.incrementIndicatorCounter();
        scenario.computeSimValues();
        scenario.doComputations();
        scenario.computePDF();

        Session.addScenario(scenario);
        return scenario;
    }

    public static Scenario createScenario(String name, double xMngValue, double yMngValue, Color color) {
        MngAction mngAction = new MngAction();
        double xMngValueRounded = Math.round(xMngValue / getMngOption(0).getRoundFactor()) * getMngOption(0).getRoundFactor();
        double yMngValueRounded = Math.round(yMngValue / getMngOption(1).getRoundFactor()) * getMngOption(1).getRoundFactor();
        mngAction.addMngValue(getMngOption(0), xMngValueRounded);
        mngAction.addMngValue(getMngOption(1), yMngValueRounded);

        Scenario scenario = new Scenario(mngAction);
        Scenario.incrementIndicatorCounter();
        scenario.setName(name);
        scenario.setColor(color);

        scenario.computeSimValues();
        scenario.doComputations();
        scenario.computePDF();

        Session.addScenario(scenario);
        return scenario;
    }
    
    public static List<Scenario> getSortedScenarios(Indicator indicator) {
        if (indicator == null)
            return scenarios;

        List<Scenario> sortedScenarios = new ArrayList<Scenario>();
        for (Scenario scn : scenarios)
            sortedScenarios.add(scn);

        for (int i = 0; i < sortedScenarios.size(); ++i) {
            for (int j = i+1; j < sortedScenarios.size(); ++j) {
                Scenario dataItem_i = sortedScenarios.get(i);
                Scenario dataItem_j = sortedScenarios.get(j);
                if (dataItem_j.lessThan(dataItem_i, indicator)) {
                    sortedScenarios.remove(i);
                    sortedScenarios.add(i, dataItem_j);
                    sortedScenarios.remove(j);
                    sortedScenarios.add(j, dataItem_i);
                }
            }
        }
        return sortedScenarios;
    }

    public static Scenario getSelectedScenario() {
        return selectedScenario;
    }

    public static void setSelectedScenario(Scenario selectedScenario) {
        Session.selectedScenario = selectedScenario;
        Vismon.getVismonInstance().getScenarioPanel().highlightSelectedScenario();
    }

    public static int getErrorBarPercentage() {
        return errorBarPercentage;
    }

    public static void setErrorBarPercentage(int errorBarPercentage) {
        Session.errorBarPercentage = errorBarPercentage;
    }

    public static int getSimulationNum() {
        return simulationNum;
    }

    public static void setSimulationNum(int simulationNum) {
        Session.simulationNum = simulationNum;
    }

    public static List<DataItem> getDataItems() {
        return dataItems;
    }

    public static void setDataItems(List<DataItem> dataItems) {
        Session.dataItems = dataItems;
    }
    
    public static void addDataItem(DataItem di) {
        dataItems.add(di);
    }

    public static void addDataItem(MngAction ic) {
        dataItems.add(new DataItem(ic));
    }

    public static Scenario getCrosshairScenario() {
        return crosshairScenario;
    }

    public static void setCrosshairScenario(Scenario crosshairScenarion) {
        Session.crosshairScenario = crosshairScenario;
    }

    public static void setCrosshairScenario(double xMngValue, double yMngValue) {
        if (!OptionsPanel.showCrosshairs && !OptionsPanel.seeMCTrials) {
            crosshairScenario = null;
            return;
        }

        MngAction mngAction = new MngAction();
        double xMngValueRounded = Math.round(xMngValue / getMngOption(0).getRoundFactor()) * getMngOption(0).getRoundFactor();
        double yMngValueRounded = Math.round(yMngValue / getMngOption(1).getRoundFactor()) * getMngOption(1).getRoundFactor();
        mngAction.addMngValue(getMngOption(0), xMngValueRounded);
        mngAction.addMngValue(getMngOption(1), yMngValueRounded);

        crosshairScenario = new Scenario(mngAction);
        crosshairScenario.computeSimValues();
        crosshairScenario.doComputations();
    }

    public static int getIndicatorColNum() {
        return indicatorColNum;
    }

    public static void setIndicatorColNum(int indicatorColNum) {
        Session.indicatorColNum = indicatorColNum;
    }

    public static int getIndicatorRowNum() {
        return indicatorRowNum;
    }

    public static void setIndicatorRowNum(int indicatorRowNum) {
        Session.indicatorRowNum = indicatorRowNum;
    }

    public static int getIndicatorRank(Indicator ind) {
        for (int i = 0; i < indicatorNum; ++i)
            if (getIndicator(i).equals(ind))
                return i;
        return -1;
    }

    public static boolean addValue(MngAction action, int simNum, Indicator dim, double newValue) {
        DataItem di = getDataItem(action);
        if (di != null) {
            di.addValue(simNum, dim, newValue);
            return true;
        }
        return false;
    }

    public static DataItem getDataItem(MngAction action) {
        for (DataItem di : dataItems)
            if (di.hasMngAction(action))
                return di;
        return null;
    }

    public static DataItem getDataItem(int simNum, Indicator dim, double v) {
        for (DataItem di : dataItems)
            if (di.hasValue(simNum, dim, v))
                return di;
        return null;
    }

    public static DataItem getDataItem(Indicator dim, double averageV) {
        for (DataItem di : dataItems)
            if (di.hasAverageValue(dim, averageV))
                return di;
        return null;
    }

    public static DataItem getDataItemBySortedAverages(Indicator dim, int rank) {
        double[] sortedAverarges = getAverageValuesOfDimension(dim);
        Arrays.sort(sortedAverarges);
        for (DataItem di : dataItems)
            if (di.hasAverageValue(dim, sortedAverarges[rank]))
                return di;
        return null;
    }

    public static DataItem getMiddleDataItem(MngAction middleIC) {
        for (MngOption indepDim : mngOptions) {
            double indepValue = middleIC.getMngValue(indepDim);
            if (indepValue < indepDim.getMin() || indepDim.getMax() < indepValue)
                return null;
        }
        
        int prev = 0;
        int next = 1;

        double[][] prevNextIndepValues = new double[mngOptionNum][2];

        int index = 0;
        for (MngOption indepDim : mngOptions) {
            prevNextIndepValues[index][prev] = indepDim.getNearestPrevValue(middleIC.getMngValue(indepDim));
            prevNextIndepValues[index][next] = indepDim.getNearestNextValue(middleIC.getMngValue(indepDim));
            ++index;
        }

        MngAction lbIC = null;
        MngAction rbIC = null;
        MngAction ltIC = null;
        MngAction rtIC = null;

        MngOption indep0 = getMngOption(0);
        MngOption indep1 = getMngOption(1);



        if (!Util.isNaN(prevNextIndepValues[0][prev]) && !Util.isNaN(prevNextIndepValues[1][prev])) {
            lbIC = new MngAction();
            lbIC.addMngValue(indep0, prevNextIndepValues[0][prev]);
            lbIC.addMngValue(indep1, prevNextIndepValues[1][prev]);
        }
        if (!Util.isNaN(prevNextIndepValues[0][next]) && !Util.isNaN(prevNextIndepValues[1][prev])) {
            rbIC = new MngAction();
            rbIC.addMngValue(indep0, prevNextIndepValues[0][next]);
            rbIC.addMngValue(indep1, prevNextIndepValues[1][prev]);
        }
        if (!Util.isNaN(prevNextIndepValues[0][prev]) && !Util.isNaN(prevNextIndepValues[1][next])) {
            ltIC = new MngAction();
            ltIC.addMngValue(indep0, prevNextIndepValues[0][prev]);
            ltIC.addMngValue(indep1, prevNextIndepValues[1][next]);
        }
        if (!Util.isNaN(prevNextIndepValues[0][next]) && !Util.isNaN(prevNextIndepValues[1][next])) {
            rtIC = new MngAction();
            rtIC.addMngValue(indep0, prevNextIndepValues[0][next]);
            rtIC.addMngValue(indep1, prevNextIndepValues[1][next]);
        }

        DataItem middleDI = new DataItem(middleIC);
        
        if (lbIC != null && rbIC != null && ltIC != null && rtIC != null) {
            /*System.out.println("\n\n");
            System.out.println("lb -> " + lbIC);
            System.out.println("rb -> " + rbIC);
            System.out.println("lt -> " + ltIC);
            System.out.println("rt -> " + rtIC);*/
            double a, b;
            if (rbIC.getMngValue(indep0) == lbIC.getMngValue(indep0))
                a = 0;
            else
                a = (middleIC.getMngValue(indep0) - lbIC.getMngValue(indep0)) / (rbIC.getMngValue(indep0) - lbIC.getMngValue(indep0));
            if (ltIC.getMngValue(indep1) == lbIC.getMngValue(indep1))
                b = 0;
            else
                b = (middleIC.getMngValue(indep1) - lbIC.getMngValue(indep1)) / (ltIC.getMngValue(indep1) - lbIC.getMngValue(indep1));
            //System.out.println("a = " + a);
            //System.out.println("b = " + b);

            DataItem lbDI = getDataItem(lbIC);
            DataItem rbDI = getDataItem(rbIC);
            DataItem ltDI = getDataItem(ltIC);
            DataItem rtDI = getDataItem(rtIC);

            for (int s = 0; s < simulationNum; ++s) {
                for (Indicator depDim : indicators) {
                    double lbV = lbDI.getValue(s, depDim);
                    double rbV = rbDI.getValue(s, depDim);
                    double ltV = ltDI.getValue(s, depDim);
                    double rtV = rtDI.getValue(s, depDim);

                    double middleV = (1 - b) * ((1 - a) * lbV + a * rbV)
                                    +   b    * ((1 - a) * ltV + a * rtV);
                    middleDI.addValue(s, depDim, middleV);
                }
            }
        } else if (lbIC != null && rbIC != null) {
            //System.out.println("\n");
            //System.out.println("lb -> " + lbIC);
            //System.out.println("rb -> " + rbIC);
            double a = (middleIC.getMngValue(indep0) - lbIC.getMngValue(indep0)) / (rbIC.getMngValue(indep0) - lbIC.getMngValue(indep0));
            //System.out.println("a = " + a);
            
            DataItem lbDI = getDataItem(lbIC);
            DataItem rbDI = getDataItem(rbIC);
            for (int s = 0; s < simulationNum; ++s) {
                for (Indicator depDim : indicators) {
                    double lbV = lbDI.getValue(s, depDim);
                    double rbV = rbDI.getValue(s, depDim);
                    
                    double middleV = (1 - a) * lbV + a * rbV;
                    middleDI.addValue(s, depDim, middleV);
                }
            }
        } else if (lbIC != null && ltIC != null) {
            //System.out.println("\n");
            //System.out.println("lb -> " + lbIC);
            //System.out.println("lt -> " + ltIC);
            double b = (middleIC.getMngValue(indep1) - lbIC.getMngValue(indep1)) / (ltIC.getMngValue(indep1) - lbIC.getMngValue(indep1));
            //System.out.println("b = " + b);

            DataItem lbDI = getDataItem(lbIC);
            DataItem ltDI = getDataItem(ltIC);
            
            for (int s = 0; s < simulationNum; ++s) {
                for (Indicator depDim : indicators) {
                    double lbV = lbDI.getValue(s, depDim);
                    double ltV = ltDI.getValue(s, depDim);
                    
                    double middleV = (1 - b) * lbV + b * ltV;
                    middleDI.addValue(s, depDim, middleV);
                }
            }
        } else {
            //System.out.println("\n");
            //System.out.println("lb -> " + lbIC);
            middleDI = getDataItem(lbIC);
        }

        return middleDI;
    }

    public static boolean isFiltered(MngAction ic) {
        for (DataItem di : dataItems)
            if (di.hasMngAction(ic))
                return di.isFiltered();
        return true;
    }

    public static void doStatisticalComputations() {
        for (DataItem di : dataItems)
            di.doComputations();
    }

    public static double[] getAverageValuesOfDimension(Indicator dim) {
        double[] averageValues = new double[dataItems.size()];
        int index = 0;
        for (DataItem di : dataItems)
            averageValues[index++] = di.getAverageValue(dim);
        return averageValues;
    }

    public static void calcIndicatorContourStep(Indicator ind) {
        double[] dataItemAvgs = new double[dataItems.size()];
        double avgMin = Double.MAX_VALUE;
        double avgMax = Double.MIN_VALUE;

        for (int i = 0; i < dataItemAvgs.length; ++i) {
            dataItemAvgs[i] = dataItems.get(i).getAverageValue(ind);
            if (dataItemAvgs[i] < avgMin)
                avgMin = dataItemAvgs[i];
            if (dataItemAvgs[i] > avgMax)
                avgMax = dataItemAvgs[i];
        }

        Computations.computeIndAvgContourStep(ind, avgMin, avgMax);
        Computations.computeIndAvgContourStepNumber(ind, avgMin, avgMax);
        Computations.computeIndAvgIsolineValues(ind, avgMin, avgMax);
    }

    public static int getScenarioRank(Scenario scenario) {
        for (int i = 0; i < scenarios.size(); ++i) {
            if (scenario.equals(scenarios.get(i)))
                return i;
        }
        return -1;
    }
}
