package vismon.gui;


import info.clearthought.layout.TableLayout;
import org.noos.xing.mydoggy.*;
import org.noos.xing.mydoggy.event.ContentManagerUIEvent;
import org.noos.xing.mydoggy.plaf.MyDoggyToolWindowManager;
import org.noos.xing.mydoggy.plaf.ui.cmp.ContentDialog;
import vismon.data.Session;
import vismon.io.FileHandler;
import vismon.plot.ScatterPlot;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class Vismon {

    private JFrame frame;

    private ToolWindowManager eastToolWindowManager;
    private ToolWindowManager westToolWindowManager;
    private ToolWindowManager southToolWindowManager;

    private JPanel northPanel;
    private JPanel eastPanel;
    private JPanel westPanel;
    private JPanel southPanel;

    private JSplitPane splitHorizontal;
    private JSplitPane splitVertical;

    private JMenu fileMenu;
    private JMenu editMenu;
    private JMenu windowMenu;
    private JMenu helpMenu;

    private JMenuItem openMenuItem;
    private JMenuItem openRecentDataMenuItem;
    private JMenuItem openProjectMenuItem;
    private JMenuItem saveProjectMenuItem;
    private JMenuItem saveProjectAsMenuItem;
    private JMenuItem exportWinMenuItem;
    private JMenuItem exportSceMenuItem;

    private JCheckBoxMenuItem overviewWinMenuItem;
    private JCheckBoxMenuItem cplomWinMenuItem;
    private JCheckBoxMenuItem tradeoffsWinMenuItem;
    private JCheckBoxMenuItem visualRefLineWinMenuItem;

    private VisualRefLineDialog visualRefLineDialog;

    private OverviewPanel overviewPanel;
    private CplomPanel cplomPanel;
    private OptionsPanel optionsPanel;
    private TradeoffsPanel tradeoffsPanel;
    private ScenariosPanel scenariosPanel;
    private IndicatorsPanel indicatorsPanel;

    private List<ScatterPlot> scatterPlotList;

    private boolean tradeoffsPanelClosed;

    protected void run() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                setUp();
                start();
            }
        });
    }

    protected void setUp() {
        overviewPanel = new OverviewPanel();
        cplomPanel = new CplomPanel();
        optionsPanel = new OptionsPanel();
        tradeoffsPanel = new TradeoffsPanel();
        scenariosPanel = new ScenariosPanel();
        indicatorsPanel = new IndicatorsPanel();

        scatterPlotList = new ArrayList<ScatterPlot>();

        tradeoffsPanelClosed = false;

        ToolTipManager.sharedInstance().setInitialDelay(10);

        initComponents();
        initSplitPane();
        initWestToolWindowManager();
        initEastToolWindowManager();
        initSouthToolWindowManager();
    }

    protected void start() {
        ToolWindow optionsTool = eastToolWindowManager.getToolWindow("Options");
        optionsTool.setVisible(false);
        optionsTool.setIndex(0);

        ToolWindow scenariosTool = southToolWindowManager.getToolWindow("Scenarios");
        scenariosTool.setVisible(false);
        scenariosTool.setIndex(0);

        ToolWindow indicatorTool = southToolWindowManager.getToolWindow("Indicators");
        indicatorTool.setVisible(false);
        indicatorTool.setIndex(0);

        setSubPanelVisibility();
        frame.setVisible(true);
    }

    protected void initComponents() {
        this.frame = new JFrame("Vismon");
        this.frame.setSize(Toolkit.getDefaultToolkit().getScreenSize());
        this.frame.setLocation(0, 0);
        this.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.frame.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent windowEvent) {
                Vismon.getVismonInstance().close();
            }
        });

        addMenus();
        setEnabledMenus(false);

        this.frame.getContentPane().setLayout(new TableLayout(new double[][]{{0, -1, 0}, {0, -1, 0}}));
    }

    private void close() {
        frame.setVisible(false);
        frame.dispose();
        System.exit(0);
    }

    private void addMenus() {
        JMenuBar menuBar = new JMenuBar();

        addFileMenu(menuBar);
        addEditMenu(menuBar);
        addWindowMenu(menuBar);
        addHelpMenu(menuBar);

        this.frame.setJMenuBar(menuBar);
    }

    private void setEnabledMenus(boolean enabled) {
        editMenu.setEnabled(enabled);
        windowMenu.setEnabled(enabled);
        helpMenu.setEnabled(enabled);

        openMenuItem.setEnabled(!enabled);
        openRecentDataMenuItem.setEnabled(!enabled);
        openProjectMenuItem.setEnabled(!enabled);

        saveProjectMenuItem.setEnabled(enabled);
        saveProjectAsMenuItem.setEnabled(enabled);
        exportWinMenuItem.setEnabled(enabled);
        exportSceMenuItem.setEnabled(enabled);
    }

    private void addFileMenu(JMenuBar menuBar) {
        final ProgressBar progressBar = new ProgressBar();
        fileMenu = new JMenu("File");

        openMenuItem = new JMenuItem("Open Data");
        openMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                final JFileChooser chooser = new JFileChooser(new File("."));
                final int response = chooser.showOpenDialog(null);
                Thread thread = new Thread() {
                    public void run() {
                        if (response == JFileChooser.APPROVE_OPTION) {
                            progressBar.run();
                            //progressBar.setStatusLabelText("Reading the input file");
                            File f = chooser.getSelectedFile();
                            FileHandler.readDataFile(f);
                        }
                        setEnabledMenus(true);
                        setSubPanelVisibility();

                        //progressBar.setStatusLabelText("Creating the Overview Panel");
                        overviewPanel.addMngOptionDualSliders();
                        overviewPanel.createIndSliderLayouts();
                        overviewPanel.createIndSubPanels();
                        overviewPanel.addIndicatorGUIs();
                        overviewPanel.repaint();

                        //progressBar.setStatusLabelText("Drawing the Contour Plots");
                        cplomPanel.createLabels();
                        cplomPanel.createPlotPanels();
                        cplomPanel.createCenterPanelLayout();
                        cplomPanel.addGUI();
                        cplomPanel.addPopupMenuItems();
                        cplomPanel.repaint();

                        //progressBar.setStatusLabelText("Creating the Trade-offs Panel");
                        tradeoffsPanel.addSortByIndComboBox();
                        tradeoffsPanel.setUncertaintyComboBoxSelected();
                        tradeoffsPanel.createIndicatorPlots();
                        tradeoffsPanel.addPlots();

                        indicatorsPanel.addIndicatorCheckBoxes();

                        progressBar.close();
                    }
                };
                thread.start();
            }
        });
        openMenuItem.setAccelerator(KeyStroke.getKeyStroke('O', InputEvent.CTRL_DOWN_MASK));
        fileMenu.add(openMenuItem);

        openRecentDataMenuItem = new JMenuItem("Open Recent Data");
        openMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("Open Recent Data menu item was pressed.");
            }
        });
        fileMenu.add(openRecentDataMenuItem);

        openProjectMenuItem = new JMenuItem("Open Project");
        openProjectMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("Open Project menu item was pressed.");
            }
        });
        fileMenu.add(openProjectMenuItem);

        fileMenu.addSeparator();

        saveProjectMenuItem = new JMenuItem("Save Project");
        saveProjectMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("Save Project menu item was pressed.");
            }
        });
        saveProjectMenuItem.setAccelerator(KeyStroke.getKeyStroke('S', InputEvent.CTRL_DOWN_MASK));
        fileMenu.add(saveProjectMenuItem);

        saveProjectAsMenuItem = new JMenuItem("Save Project as");
        saveProjectAsMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("Save Project As menu item was pressed.");
            }
        });
        saveProjectAsMenuItem.setAccelerator(KeyStroke.getKeyStroke('S', InputEvent.CTRL_DOWN_MASK | InputEvent.SHIFT_DOWN_MASK));
        fileMenu.add(saveProjectAsMenuItem);

        fileMenu.addSeparator();

        exportWinMenuItem = new JMenuItem("Export Windows to PNG");
        exportWinMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("Export Windows to PNG menu item was pressed.");
            }
        });
        fileMenu.add(exportWinMenuItem);

        exportSceMenuItem = new JMenuItem("Export Scenarios to CSV");
        exportSceMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("Export Scenarios to PNG menu item was pressed.");
            }
        });
        fileMenu.add(exportSceMenuItem);

        fileMenu.addSeparator();

        JMenuItem exitMenuItem = new JMenuItem("Exit");
        exitMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                close();
            }
        });
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);
    }

    private void addEditMenu(JMenuBar menuBar) {
        editMenu = new JMenu("Edit");

        JMenuItem undoMenuItem = new JMenuItem("Undo");
        undoMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("Undo menu item was pressed.");
            }
        });
        undoMenuItem.setAccelerator(KeyStroke.getKeyStroke('Z', InputEvent.CTRL_DOWN_MASK));
        editMenu.add(undoMenuItem);

        editMenu.addSeparator();

        JMenuItem deleteAllSceMenuItem = new JMenuItem("Delete all Scenarios");
        deleteAllSceMenuItem.setIcon(new ImageIcon("images/delete.png"));
        deleteAllSceMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scenariosPanel.emptyTable();
                Session.removeAllScenarios();
            }
        });
        editMenu.add(deleteAllSceMenuItem);

        editMenu.addSeparator();

        JMenuItem preferencesMenuItem = new JMenuItem("Preferences");
        preferencesMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Preferences preferences = new Preferences();
                preferences.run();
            }
        });
        editMenu.add(preferencesMenuItem);

        menuBar.add(editMenu);
    }

    private void addWindowMenu(JMenuBar menuBar) {
        windowMenu = new JMenu("Window");

        overviewWinMenuItem = new JCheckBoxMenuItem("Overview");
        overviewWinMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (overviewWinMenuItem.isSelected() &&
                    westToolWindowManager.getContentManager().getContentCount() == 0)
                    initWestToolWindowManager();
                overviewWinMenuItem.setSelected(overviewWinMenuItem.isSelected());
                setSubPanelVisibility();
            }
        });
        overviewWinMenuItem.setSelected(true);
        windowMenu.add(overviewWinMenuItem);

        cplomWinMenuItem = new JCheckBoxMenuItem("Contour Plots");
        cplomWinMenuItem.setSelected(true);
        cplomWinMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (cplomWinMenuItem.isSelected() &&
                    eastToolWindowManager.getContentManager().getContentCount() == 0)
                    initEastToolWindowManager();
                cplomWinMenuItem.setSelected(cplomWinMenuItem.isSelected());
                setSubPanelVisibility();
            }
        });
        cplomWinMenuItem.setSelected(true);
        windowMenu.add(cplomWinMenuItem);

        tradeoffsWinMenuItem = new JCheckBoxMenuItem("Trade-offs");
        tradeoffsWinMenuItem.setSelected(true);
        tradeoffsWinMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (tradeoffsWinMenuItem.isSelected() &&
                    southToolWindowManager.getContentManager().getContentCount() == 0)
                    initSouthToolWindowManager();
                tradeoffsWinMenuItem.setSelected(tradeoffsWinMenuItem.isSelected());
                setSubPanelVisibility();
            }
        });
        tradeoffsWinMenuItem.setSelected(false);
        windowMenu.add(tradeoffsWinMenuItem);

        visualRefLineWinMenuItem = new JCheckBoxMenuItem("Visual Reference Lines");
        visualRefLineWinMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (visualRefLineWinMenuItem.isSelected()) {
                    visualRefLineDialog = new VisualRefLineDialog();
                    visualRefLineDialog.run();
                } else {
                    visualRefLineDialog.close();
                    visualRefLineDialog = null;
                }
            }
        });
        visualRefLineWinMenuItem.setSelected(false);
        windowMenu.add(visualRefLineWinMenuItem);

        menuBar.add(windowMenu);
    }

    private void addHelpMenu(JMenuBar menuBar) {
        helpMenu = new JMenu("Help");

        JMenuItem HelpMenuItem = new JMenuItem("Help");
        HelpMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("Help menu item was pressed.");
            }
        });
        helpMenu.add(HelpMenuItem);

        JMenuItem aboutMenuItem = new JMenuItem("About");
        aboutMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("About menu item was pressed.");
            }
        });
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);
    }

    protected void initSplitPane() {
        splitHorizontal = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitHorizontal.setBorder(BorderFactory.createEmptyBorder());
        splitHorizontal.setOneTouchExpandable(false);
        splitHorizontal.setDividerSize(4);
        splitHorizontal.setContinuousLayout(true);
        splitHorizontal.setDividerLocation((int)(this.frame.getHeight() * 0.65));

        northPanel = new JPanel(new BorderLayout());
        splitHorizontal.setTopComponent(northPanel);

        southPanel = new JPanel(new BorderLayout());
        splitHorizontal.setBottomComponent(southPanel);

        Container frameContainer = this.frame.getContentPane();
        frameContainer.setLayout(new BorderLayout());
        frameContainer.add(splitHorizontal, BorderLayout.CENTER);

        splitVertical = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        splitVertical.setBorder(BorderFactory.createEmptyBorder());
        splitVertical.setOneTouchExpandable(false);
        splitVertical.setDividerSize(4);
        splitVertical.setContinuousLayout(true);
        splitVertical.setDividerLocation((int) (this.frame.getWidth() * 0.35));

        eastPanel = new JPanel(new BorderLayout());
        splitVertical.setRightComponent(eastPanel);

        westPanel = new JPanel(new BorderLayout());
        splitVertical.setLeftComponent(westPanel);
        northPanel.add(splitVertical, BorderLayout.CENTER);
    }

    protected void initWestToolWindowManager() {
        MyDoggyToolWindowManager myDoggyToolWindowManager = new MyDoggyToolWindowManager();
        this.westToolWindowManager = myDoggyToolWindowManager;

        initWestContentManager();

        westPanel.add(myDoggyToolWindowManager);
    }

    protected void initEastToolWindowManager() {
        MyDoggyToolWindowManager myDoggyToolWindowManager = new MyDoggyToolWindowManager();
        this.eastToolWindowManager = myDoggyToolWindowManager;

        // Register a Tool.

        eastToolWindowManager.registerToolWindow("Options",                      // Id
                                             "Options",                 // Title
                                             null,                         // Icon
                                             optionsPanel,    // Component
                                             ToolWindowAnchor.RIGHT);       // Anchor

        setupEastTool();

        for (ToolWindow window : eastToolWindowManager.getToolWindows())
            window.setAvailable(true);

        initEastContentManager();

        eastPanel.add(myDoggyToolWindowManager);
    }

    protected void initSouthToolWindowManager() {
        MyDoggyToolWindowManager myDoggyToolWindowManager = new MyDoggyToolWindowManager();
        this.southToolWindowManager = myDoggyToolWindowManager;

        // Register a Tool.

        southToolWindowManager.registerToolWindow("Indicators",                      // Id
                                             "Indicators",                 // Title
                                             null,                         // Icon
                                             indicatorsPanel,    // Component
                                             ToolWindowAnchor.RIGHT);       // Anchor

        southToolWindowManager.registerToolWindow("Scenarios",                      // Id
                                             "Scenarios",                 // Title
                                             null,                         // Icon
                                             scenariosPanel,    // Component
                                             ToolWindowAnchor.RIGHT);       // Anchor

        setupSouthTool();

        for (ToolWindow window : southToolWindowManager.getToolWindows())
            window.setAvailable(true);

        initSouthContentManager();

        southPanel.add(myDoggyToolWindowManager);
    }

    protected void setupEastTool() {
        ToolWindow optionsTool = eastToolWindowManager.getToolWindow("Options");

        // RepresentativeAnchorDescriptor
        RepresentativeAnchorDescriptor representativeAnchorDescriptor = optionsTool.getRepresentativeAnchorDescriptor();
        representativeAnchorDescriptor.setPreviewEnabled(true);
        representativeAnchorDescriptor.setPreviewDelay(1500);
        representativeAnchorDescriptor.setPreviewTransparentRatio(0.4f);

        // DockedTypeDescriptor
        DockedTypeDescriptor dockedTypeDescriptor = (DockedTypeDescriptor) optionsTool.getTypeDescriptor(ToolWindowType.DOCKED);
        dockedTypeDescriptor.setDockLength(300);
        dockedTypeDescriptor.setPopupMenuEnabled(true);
        /*JMenu toolsMenu = dockedTypeDescriptor.getToolsMenu();
        toolsMenu.add(new AbstractAction("Hello World!!!") {
            public void actionPerformed(ActionEvent e) {
                JOptionPane.showMessageDialog(frame, "Hello World!!!");
            }
        });
        dockedTypeDescriptor.setToolWindowActionHandler(new ToolWindowActionHandler() {
            public void onHideButtonClick(ToolWindow toolWindow) {
                JOptionPane.showMessageDialog(frame, "Hiding...");
                toolWindow.setVisible(false);
            }
        });*/
        //dockedTypeDescriptor.setAnimating(true);

        // SlidingTypeDescriptor
        SlidingTypeDescriptor slidingTypeDescriptor = (SlidingTypeDescriptor) optionsTool.getTypeDescriptor(ToolWindowType.SLIDING);
        slidingTypeDescriptor.setEnabled(false);

        // FloatingTypeDescriptor
        FloatingTypeDescriptor floatingTypeDescriptor = (FloatingTypeDescriptor) optionsTool.getTypeDescriptor(ToolWindowType.FLOATING);
        floatingTypeDescriptor.setEnabled(false);

    }

    protected void setupSouthTool() {
        ToolWindow scenariosTool = southToolWindowManager.getToolWindow("Scenarios");

        // RepresentativeAnchorDescriptor
        RepresentativeAnchorDescriptor representativeAnchorDescriptor = scenariosTool.getRepresentativeAnchorDescriptor();
        representativeAnchorDescriptor.setPreviewEnabled(true);
        representativeAnchorDescriptor.setPreviewDelay(1500);
        representativeAnchorDescriptor.setPreviewTransparentRatio(0.4f);

        // DockedTypeDescriptor
        DockedTypeDescriptor dockedTypeDescriptor = (DockedTypeDescriptor) scenariosTool.getTypeDescriptor(ToolWindowType.DOCKED);
        dockedTypeDescriptor.setDockLength(300);
        dockedTypeDescriptor.setPopupMenuEnabled(true);
        dockedTypeDescriptor.setAnimating(true);

        // SlidingTypeDescriptor
        SlidingTypeDescriptor slidingTypeDescriptor = (SlidingTypeDescriptor) scenariosTool.getTypeDescriptor(ToolWindowType.SLIDING);
        slidingTypeDescriptor.setEnabled(false);

        // FloatingTypeDescriptor
        FloatingTypeDescriptor floatingTypeDescriptor = (FloatingTypeDescriptor) scenariosTool.getTypeDescriptor(ToolWindowType.FLOATING);
        floatingTypeDescriptor.setEnabled(false);

        //////////////////

        ToolWindow indTool = southToolWindowManager.getToolWindow("Indicators");

        // RepresentativeAnchorDescriptor
        RepresentativeAnchorDescriptor representativeAnchorDescriptor2 = indTool.getRepresentativeAnchorDescriptor();
        representativeAnchorDescriptor2.setPreviewEnabled(true);
        representativeAnchorDescriptor2.setPreviewDelay(1500);
        representativeAnchorDescriptor2.setPreviewTransparentRatio(0.4f);

        // DockedTypeDescriptor
        DockedTypeDescriptor dockedTypeDescriptor2 = (DockedTypeDescriptor) indTool.getTypeDescriptor(ToolWindowType.DOCKED);
        dockedTypeDescriptor2.setDockLength(300);
        dockedTypeDescriptor2.setPopupMenuEnabled(true);
        dockedTypeDescriptor2.setAnimating(true);

        // SlidingTypeDescriptor
        SlidingTypeDescriptor slidingTypeDescriptor2 = (SlidingTypeDescriptor) indTool.getTypeDescriptor(ToolWindowType.SLIDING);
        slidingTypeDescriptor2.setEnabled(false);

        // FloatingTypeDescriptor
        FloatingTypeDescriptor floatingTypeDescriptor2 = (FloatingTypeDescriptor) indTool.getTypeDescriptor(ToolWindowType.FLOATING);
        floatingTypeDescriptor2.setEnabled(false);

    }

    protected void initWestContentManager() {
        ContentManager contentManager = westToolWindowManager.getContentManager();
        Content content = contentManager.addContent("Overview",
                                                    "Overview",
                                                    null,      // An icon
                                                    overviewPanel);
        content.setToolTipText("Overview of Management Options and Indicators");

        setupWestContentManagerUI();
    }

    protected void initEastContentManager() {
        ContentManager contentManager = eastToolWindowManager.getContentManager();
        Content content = contentManager.addContent("Contour Plot Matrix",
                                                    "Contour Plot Matrix",
                                                    null,      // An icon
                                                    cplomPanel);
        content.setToolTipText("Contour Plot Matrix of Indicators");

        setupEastContentManagerUI();
    }

    protected void initSouthContentManager() {
        ContentManager contentManager = southToolWindowManager.getContentManager();
        Content content = contentManager.addContent("Trade-offs",
                                                    "Trade-offs",
                                                    null,      // An icon
                                                    tradeoffsPanel);
        content.setToolTipText("Trade-offs");
        setupSouthContentManagerUI();
    }

    protected void setupWestContentManagerUI() {

        TabbedContentManagerUI contentManagerUI = (TabbedContentManagerUI) westToolWindowManager.getContentManager().getContentManagerUI();

        contentManagerUI.setShowAlwaysTab(true);
        contentManagerUI.setTabPlacement(TabbedContentManagerUI.TabPlacement.TOP);
        contentManagerUI.addContentManagerUIListener(new ContentManagerUIListener() {
            public boolean contentUIRemoving(ContentManagerUIEvent event) {
                //splitVertical.setDividerLocation(0);
                System.out.println("closing overviewPanel");
                overviewWinMenuItem.setSelected(false);
                setSubPanelVisibility();
                return true;
            }

            public void contentUIDetached(ContentManagerUIEvent event) {
                splitVertical.setDividerLocation(0);
            }
        });

        TabbedContentUI contentUI = (TabbedContentUI) westToolWindowManager.getContentManager().getContent(0).getContentUI();

        contentUI.setCloseable(false);
        contentUI.setMaximizable(false);
        contentUI.setDetachable(true);
        contentUI.setMinimizable(false);
        contentUI.setTransparentMode(true);
        contentUI.setTransparentRatio(0.7f);
        contentUI.setTransparentDelay(1000);

        final Content content = westToolWindowManager.getContentManager().getContent(0);
        content.addPropertyChangeListener("detached", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                if (propertyChangeEvent.getNewValue().equals(Boolean.TRUE)) {
                    Container container = content.getComponent().getParent();
                    while (!(container instanceof ContentDialog))
                        container = container.getParent();
                    ContentDialog dialog = (ContentDialog) container;
                    dialog.setLocation(0, 0);
                    dialog.setSize(Toolkit.getDefaultToolkit().getScreenSize());
                } else {
                    splitVertical.setDividerLocation((int) (frame.getWidth() * 0.35));
                }
            }
        });
    }

    protected void setupEastContentManagerUI() {

        TabbedContentManagerUI contentManagerUI = (TabbedContentManagerUI) eastToolWindowManager.getContentManager().getContentManagerUI();

        contentManagerUI.setShowAlwaysTab(true);
        contentManagerUI.setTabPlacement(TabbedContentManagerUI.TabPlacement.TOP);
        contentManagerUI.addContentManagerUIListener(new ContentManagerUIListener() {
            public boolean contentUIRemoving(ContentManagerUIEvent event) {
                System.out.println("closing cplomPanel");
                //splitVertical.setDividerLocation(frame.getWidth());
                cplomWinMenuItem.setSelected(false);
                setSubPanelVisibility();
                return true;
            }

            public void contentUIDetached(ContentManagerUIEvent event) {
                splitVertical.setDividerLocation(frame.getWidth());
            }
        });

        TabbedContentUI contentUI = (TabbedContentUI) eastToolWindowManager.getContentManager().getContent(0).getContentUI();

        contentUI.setCloseable(false);
        contentUI.setMaximizable(false);
        contentUI.setDetachable(true);
        contentUI.setMinimizable(false);
        contentUI.setTransparentMode(true);
        contentUI.setTransparentRatio(0.7f);
        contentUI.setTransparentDelay(1000);

        final Content content = eastToolWindowManager.getContentManager().getContent(0);
        content.addPropertyChangeListener("detached", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                if (propertyChangeEvent.getNewValue().equals(Boolean.TRUE)) {
                    Container container = content.getComponent().getParent();
                    while (!(container instanceof ContentDialog))
                        container = container.getParent();
                    ContentDialog dialog = (ContentDialog) container;
                    dialog.setLocation(0, 0);
                    dialog.setSize(Toolkit.getDefaultToolkit().getScreenSize());
                } else {
                    splitVertical.setDividerLocation((int) (frame.getWidth() * 0.35));
                }
            }
        });
    }

    protected void setupSouthContentManagerUI() {
        tradeoffsPanelClosed = false;

        TabbedContentManagerUI contentManagerUI = (TabbedContentManagerUI) southToolWindowManager.getContentManager().getContentManagerUI();

        contentManagerUI.setShowAlwaysTab(true);
        contentManagerUI.setTabPlacement(TabbedContentManagerUI.TabPlacement.TOP);
        contentManagerUI.addContentManagerUIListener(new ContentManagerUIListener() {
            public boolean contentUIRemoving(ContentManagerUIEvent event) {
                //splitHorizontal.setDividerLocation(frame.getHeight());
                System.out.println("closing tradeoffsPanel");
                tradeoffsWinMenuItem.setSelected(false);
                setSubPanelVisibility();
                tradeoffsPanelClosed = true;
                return true;
            }

            public void contentUIDetached(ContentManagerUIEvent event) {
                splitHorizontal.setDividerLocation(frame.getHeight());
            }
        });

        TabbedContentUI contentUI = (TabbedContentUI) southToolWindowManager.getContentManager().getContent(0).getContentUI();

        contentUI.setCloseable(true);
        contentUI.setMaximizable(false);
        contentUI.setDetachable(true);
        contentUI.setMinimizable(false);
        contentUI.setTransparentMode(true);
        contentUI.setTransparentRatio(0.7f);
        contentUI.setTransparentDelay(1000);

        final Content content = southToolWindowManager.getContentManager().getContent(0);
        content.addPropertyChangeListener("detached", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                if (propertyChangeEvent.getNewValue().equals(Boolean.TRUE)) {
                    Container container = content.getComponent().getParent();
                    while (!(container instanceof ContentDialog))
                        container = container.getParent();
                    ContentDialog dialog = (ContentDialog) container;
                    dialog.setLocation(0, 0);
                    dialog.setSize(Toolkit.getDefaultToolkit().getScreenSize());
                    //tradeoffsPanel.setMaximized(true);
                } else {
                    splitHorizontal.setDividerLocation((int) (frame.getHeight() * 0.65));
                    //tradeoffsPanel.setMaximized(false);
                }
                tradeoffsPanel.addPlots();
            }
        });
    }

    private void setSubPanelVisibility() {
        if (tradeoffsPanelClosed)
            setupSouthContentManagerUI();

        if (openMenuItem.isEnabled()) {         // before opening a file
            northPanel.setVisible(false);
            westPanel.setVisible(false);
            eastPanel.setVisible(false);
            southPanel.setVisible(false);
            return;
        }

        double verSplitPosition;
        double horSplitPosition;

        northPanel.setVisible(true);
        westPanel.setVisible(true);
        eastPanel.setVisible(true);
        southPanel.setVisible(true);

        if (overviewWinMenuItem.isSelected()) {
            if (cplomWinMenuItem.isSelected()) {
                if (tradeoffsWinMenuItem.isSelected()) {            // overview, cplom, trade-offs
                    verSplitPosition = 0.35;
                    horSplitPosition = 0.65;
                } else {                                            // overview, cplom
                    verSplitPosition = 0.35;
                    horSplitPosition = 1.0;

                    southPanel.setVisible(false);
                }
            } else {
                if (tradeoffsWinMenuItem.isSelected()) {            // overview, trade-offs
                    verSplitPosition = 1.0;
                    horSplitPosition = 0.65;

                    eastPanel.setVisible(false);
                } else {                                            // overview
                    verSplitPosition = 1.0;
                    horSplitPosition = 1.0;

                    eastPanel.setVisible(false);
                    southPanel.setVisible(false);
                }
            }
        } else {
            if (cplomWinMenuItem.isSelected()) {
                if (tradeoffsWinMenuItem.isSelected()) {            // cplom, trade-offs
                    verSplitPosition = 0.0;
                    horSplitPosition = 0.65;

                    westPanel.setVisible(false);
                } else {                                            // cplom
                    verSplitPosition = 0.0;
                    horSplitPosition = 1.0;

                    westPanel.setVisible(false);
                    southPanel.setVisible(false);
                }
            } else {
                if (tradeoffsWinMenuItem.isSelected()) {            // trade-offs
                    verSplitPosition = 1.0;
                    horSplitPosition = 0.0;

                    northPanel.setVisible(false);
                } else {                                            // none
                    verSplitPosition = 1.0;
                    horSplitPosition = 0.0;

                    northPanel.setVisible(false);
                    southPanel.setVisible(false);
                }
            }
        }

        splitVertical.setDividerLocation((int) (frame.getWidth() * verSplitPosition));
        splitHorizontal.setDividerLocation((int) (frame.getHeight() * horSplitPosition));
    }

    public void closeVisualRefLineDialog() {
        visualRefLineDialog.close();
        visualRefLineDialog = null;

        this.visualRefLineWinMenuItem.setSelected(false);
    }

    public void setPreferences() {
        System.out.println("preferences set!");
    }

    private static Vismon vismonInstance;

    public static Vismon getVismonInstance() {
        if (vismonInstance == null) {
            vismonInstance = new Vismon();
        }
        return vismonInstance;
    }

    public CplomPanel getCplomPanel() {
        return cplomPanel;
    }

    public OverviewPanel getOverviewPanel() {
        return overviewPanel;
    }

    public ScenariosPanel getScenarioPanel() {
        return scenariosPanel;
    }

    public TradeoffsPanel getTradeoffsPanel() {
        return tradeoffsPanel;
    }

    public IndicatorsPanel getIndicatorsPanel() {
        return indicatorsPanel;
    }

    public void repaintCplomPanel() {
        cplomPanel.repaint();
        cplomPanel.repaintAllGUIs();
    }

    public void repaintOptionsPanel() {
        optionsPanel.repaint();
    }

    public void repaintOverviewPanel() {
        overviewPanel.repaint();
    }

    public void repaintTradeoffsPanel() {
        southPanel.repaint();
    }

    public void setTradeoffsWinMenuItemSelected() {
        tradeoffsWinMenuItem.setSelected(true);
        setSubPanelVisibility();
    }

    public void updateIndicatorVisibility() {
        for (int rank = 0; rank < Session.getIndicatorNum(); ++rank) {
            boolean visible = Session.getIndicator(rank).isVisible();
            cplomPanel.setContourPlotVisible(rank, visible);
            cplomPanel.setPlotMenuItemVisible(rank, !visible);
            indicatorsPanel.setIndicatorCBoxSelected(rank, visible);
            indicatorsPanel.indicatorCBoxSelectedChanged();
        }
        tradeoffsPanel.repaint();
    }

    public void addScatterPlot(ScatterPlot scatterPlot) {
        scatterPlotList.add(scatterPlot);
    }

    public void removeScatterPlot(ScatterPlot scatterPlot) {
        scatterPlotList.remove(scatterPlot);
    }

    public void updatePlots() {
        this.frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        cplomPanel.repaintAllGUIs();
        overviewPanel.repaint();
        scenariosPanel.repaint();
        tradeoffsPanel.addPlots();
        southPanel.repaint();

        for (ScatterPlot sp : scatterPlotList)
            sp.repaint();
        this.frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }

    public static void main(String[] args) {
        System.out.println("Page " + (int)(Math.random() * 604));

        UIManager.put("swing.boldMetal", Boolean.FALSE);
        vismonInstance = new Vismon();
        try {
            vismonInstance.run();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
