package vismon.plot;

import vismon.data.Indicator;
import vismon.data.Scenario;
import vismon.data.Session;
import vismon.gui.OptionsPanel;
import vismon.gui.TradeoffsPanel;
import vismon.util.Constants;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.*;

public class BarPlotScenario extends JPanel{

    private final File DOWN_FILE = new File("images/arrow_down.png");
    private final File UP_FILE = new File("images/arrow_up.png");
    private final int UP_DOWN_HALF_WIDTH = 5;

    private final int leftBlankSpace = 10;
    private final int bottomBlankSpace = 5;

    int barWidth;

    private int x;
    private int y;
    private int w;
    private int h;

    private Scenario scenario;

    public BarPlotScenario(Scenario scenario) {
        this.scenario = scenario;

        x = leftBlankSpace;
        y = leftBlankSpace;
    }

    @Override
    public void paint(Graphics g) {
        if (!scenario.isVisible())
            return;

        w = this.getWidth() - 2 * leftBlankSpace;
        h = this.getHeight() - leftBlankSpace - bottomBlankSpace;

        drawBars((Graphics2D) g);
        drawAxis((Graphics2D) g);
        drawSortByIndicatorBaseLive((Graphics2D) g);
    }

    private void drawAxis(Graphics2D g) {
        g.setColor(Constants.MAIN_COLOR);
        g.drawLine(x, y, x, y+h);
        g.drawLine(x, y+h, x+w, y+h);
    }

    private void drawSortByIndicatorBaseLive(Graphics2D g) {
        if (TradeoffsPanel.sortByIndicator == null)
            return;

        int rank = Session.getIndicatorRank(TradeoffsPanel.sortByIndicator);
        g.setStroke(new BasicStroke(2.0f));
        g.drawRect(x+rank*barWidth, y, barWidth, h);
        g.setStroke(new BasicStroke(1.0f));
    }

    private void drawBars(Graphics2D g) {
        int barNum = Session.getIndicatorNum();
        barWidth = (int) (((double) w) / barNum);

        for (int i = 0; i < barNum; ++i) {
            Indicator ithIndicator = Session.getIndicator(i);
            if (!ithIndicator.isVisible())
                continue;

            double min = ithIndicator.getMin();
            double max = ithIndicator.getMax();

            if (TradeoffsPanel.uncertainty != TradeoffsPanel.DISTRIBUTION) {
                double barValue = scenario.getAverageOrMedianValue(ithIndicator, OptionsPanel.contoursBasedOn);
                int barHeight = (int) (h*(barValue-min)/(max-min));

                g.setColor(scenario.getColor());
                g.fillRect(x+i*barWidth, y+h-barHeight, barWidth, barHeight);

                // DRAWING THE BORDER OF THE BAR TO INDICATER SORT_SCENARIOS_BY INDICATOR
                /*if (TradeoffsPanel.sortByIndicator != null && ithIndicator.equals(TradeoffsPanel.sortByIndicator)) {
                    g.setColor(Constants.MAIN_COLOR);
                    g.drawRect(x+i*barWidth, y+h-barHeight, barWidth, barHeight);
                }*/

                if (TradeoffsPanel.uncertainty == TradeoffsPanel.ERROR_BAR) {
                    int percent = Session.getErrorBarPercentage();
                    double[] errors = scenario.getErrorBar(ithIndicator,  percent);

                    int barMin = (int) (h*(errors[0]-min)/(max-min));
                    int barMax = (int) (h*(errors[1]-min)/(max-min));

                    g.setColor(Constants.MAIN_COLOR);
                    int errorLineX = x + (int) ((i + 0.5) * barWidth);
                    g.setStroke(new BasicStroke(2.0f));
                    g.drawLine(errorLineX, y+h-barMax, errorLineX, y+h-barMin);
                    g.setStroke(new BasicStroke(1.0f));
                }
            } else {
                java.util.List<Double> pdfValues = scenario.getPdfValues().get(ithIndicator);
                double maxValue = scenario.getPdfMax().get(ithIndicator);
                int binNum = pdfValues.size();
                double[] scenarioMinMax = scenario.getErrorBar(ithIndicator, 100);
                for (int j = 0; j < binNum-1; ++j) {
                    int grayScale = (int)(255*(1-pdfValues.get(j)/maxValue));
                    Color color = new Color(grayScale, grayScale, grayScale);

                    double barMinValue = scenarioMinMax[0] + (j*1.0/binNum) * (scenarioMinMax[1]-scenarioMinMax[0]);
                    double barMaxValue = scenarioMinMax[0] + ((j+1)*1.0/binNum) * (scenarioMinMax[1]-scenarioMinMax[0]);

                    int barMin = (int) (h*(barMinValue-min)/(max-min));
                    int barMax = (int) (h*(barMaxValue-min)/(max-min));

                    g.setColor(color);
                    g.fillRect(x+i*barWidth, y+h-barMax, barWidth, barMax-barMin);
                }

                // DRAWING THE BORDER OF THE BAR TO INDICATER SORT_SCENARIOS_BY INDICATOR
                /*if (TradeoffsPanel.sortByIndicator != null && ithIndicator.equals(TradeoffsPanel.sortByIndicator)) {
                    int barScenarioMin = (int) (h*(scenarioMinMax[0]-min)/(max-min));
                    int barScenarioMax = (int) (h*(scenarioMinMax[1]-min)/(max-min));
                    g.setColor(Constants.MAIN_COLOR);
                    g.drawRect(x+i*barWidth, y+h-barScenarioMax, barWidth, barScenarioMax-barScenarioMin);
                }*/
            }


            try {
                if (ithIndicator.isMaxBest())
                    g.drawImage(ImageIO.read(new File("images/arrow_up.png")), (int)(x+(i+0.5)*barWidth - UP_DOWN_HALF_WIDTH), y, null);
                else
                    g.drawImage(ImageIO.read(new File("images/arrow_down.png")), (int)(x+(i+0.5)*barWidth - UP_DOWN_HALF_WIDTH), y, null);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public void setScenario(Scenario scenario) {
        this.scenario = scenario;
    }

    public void showToolTip(int xMouse, int yMouse) {
        int barNum = Session.getIndicatorNum();

        boolean toolTipSet = false;
        for (int i = 0; i < barNum; ++i) {
            if (y <= yMouse && yMouse <= y+h &&
                x+i*barWidth <= xMouse && xMouse < x+(i+1)*barWidth) {
                setToolTipText(Session.getIndicator(i).getShortName());
                toolTipSet = true;
                break;
            }
        }

        if (!toolTipSet)
            setToolTipText("");
    }

}
