package vismon.plot;

import vismon.data.Indicator;
import vismon.data.Scenario;
import vismon.data.Session;
import vismon.gui.OverviewPanel;
import vismon.gui.Vismon;
import vismon.util.Constants;
import vismon.util.Util;

import javax.swing.*;
import java.awt.*;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;

public class IndicatorOverviewPlot extends HistogramPlot {

    private Indicator indicator;
    private List<Integer> indicatorBinHeights;

    public IndicatorOverviewPlot(Indicator indicator) {
        this.indicator = indicator;
        min = indicator.getMin();
        max = indicator.getMax();

        int dataItemNum = Session.getDataItems().size();
        int simulationNum = Session.getSimulationNum();
        List<Double> indicatorValues = new ArrayList<Double>();
        for (int i = 0; i < simulationNum; ++i)
            for (int j = 0; j < dataItemNum; ++j) {
                double v = Session.getDataItems().get(j).getValue(i, indicator);
                if (!Util.isNaN(v))
                    indicatorValues.add(v);
            }
        indicatorBinHeights = getPDFBinHeights(indicatorValues);

        x = 0;
        y = 0;
        h = OverviewPanel.HIST_PLOT_HEIGHT;
    }

    @Override
    public void paint(Graphics g) {
        setBinHeights();
        super.paint(g);
    }

    private void setBinHeights() {
        Scenario scenario = Session.getSelectedScenario();

        if (scenario == null) {
            color = Color.lightGray;
            if (Vismon.getVismonInstance().getOverviewPanel().getDensityFunc() == OverviewPanel.PDF)
                binHeights = indicatorBinHeights;
            else
                binHeights = getCDFBinHeights(indicatorBinHeights);
        } else {
            List<Double> scenarioValues = new ArrayList<Double>();
            for (int i = 0; i < Session.getSimulationNum(); ++i)
                scenarioValues.add(scenario.getValue(i, indicator));

            color = scenario.getColor();
            if (Vismon.getVismonInstance().getOverviewPanel().getDensityFunc() == OverviewPanel.PDF)
                binHeights = getPDFBinHeights(scenarioValues);
            else
                binHeights = getCDFBinHeights(getPDFBinHeights(scenarioValues));
        }
    }
}