package vismon.plot;

import vismon.data.Indicator;
import vismon.data.Scenario;
import vismon.data.Session;
import vismon.gui.OptionsPanel;
import vismon.gui.TradeoffsPanel;
import vismon.util.Constants;
import vismon.util.Util;

import javax.swing.*;
import java.awt.*;
import java.util.HashMap;
import java.util.Map;

public class MultipodesIndicator extends JPanel {

    private final double podRadiusProportion = 0.5;
    private final double holeRadiusProportion = 0.33;

    private final double angelDif = Math.PI/20;

    private int x;
    private int y;
    private int w;
    private int h;
    private int plotLength;
    private double podLength;

    private int xCentre;
    private int yCentre;

    private Indicator indicator;

    private double min;
    private double max;

    //private Map<Scenario, Double> scenarioAngles;

    public MultipodesIndicator(Indicator indicator) {
        this.indicator = indicator;

        min = indicator.getMin();
        max = indicator.getMax();

        //scenarioAngles = new HashMap<Scenario, Double>();

        x = 0;
        y = 0;
    }

    @Override
    public void paint(Graphics g) {
        int scenarioNum = Session.getScenarios().size();
        if (scenarioNum == 0)
            return;

        w = this.getWidth();
        h = this.getHeight();

        plotLength = w;
        if (w > h)
            plotLength = h;

        drawPods((Graphics2D) g);
        drawBorder((Graphics2D) g);
    }

    private void drawPods(Graphics2D g) {
        int scenarioNum = Session.getScenarios().size();

        xCentre = x + (int) (0.5 * w);
        yCentre = y + (int) (0.5 * h);

        double angle = 2 * Math.PI / scenarioNum;
        double podRadius = podRadiusProportion * plotLength;
        double holeRadius = holeRadiusProportion * podRadius;
        podLength = (podRadius - holeRadius);

        int[] basePolygonXCoords = new int[scenarioNum];
        int[] basePolygonYCoords = new int[scenarioNum];
        for (int m = 0; m < scenarioNum; ++m) {
            basePolygonXCoords[m] = xCentre + (int) (holeRadius * Math.cos(m * angle - 0.5 * angle));
            basePolygonYCoords[m] = yCentre + (int) (holeRadius * Math.sin(m * angle - 0.5 * angle));
        }

        for (int i = 0; i < scenarioNum; ++i) {
            Scenario ithScenario = Session.getSortedScenarios(TradeoffsPanel.sortByIndicator).get(i);

            double ownAngle = /*getAngle(ithScenario);*/i * 2 * Math.PI / scenarioNum;
            int nextIndex = (i + 1) % scenarioNum;

            //DRAWING THE LINES OF EACH POD
            int outerXCoords0 = basePolygonXCoords[i] + (int) (podLength * Math.cos(ownAngle));
            int outerYCoords0 = basePolygonYCoords[i] + (int) (podLength * Math.sin(ownAngle));
            int outerXCoords1 = basePolygonXCoords[nextIndex] + (int) (podLength * Math.cos(ownAngle));
            int outerYCoords1 = basePolygonYCoords[nextIndex] + (int) (podLength * Math.sin(ownAngle));
            g.setColor(Color.LIGHT_GRAY);
            g.drawLine(basePolygonXCoords[i], basePolygonYCoords[i], outerXCoords0, outerYCoords0);
            g.drawLine(basePolygonXCoords[nextIndex], basePolygonYCoords[nextIndex], outerXCoords1, outerYCoords1);

            if (!ithScenario.isVisible())
                continue;

            if (TradeoffsPanel.uncertainty != TradeoffsPanel.DISTRIBUTION) {
                double scenarioValue = ithScenario.getAverageOrMedianValue(indicator, OptionsPanel.contoursBasedOn);
                double portion = (scenarioValue - min) / (max - min);

                int[] coloredPodXCoords = new int[4];
                int[] coloredPodYCoords = new int[4];

                coloredPodXCoords[0] = basePolygonXCoords[i];
                coloredPodXCoords[1] = basePolygonXCoords[i] + (int) (portion * podLength * Math.cos(ownAngle));
                coloredPodXCoords[2] = basePolygonXCoords[nextIndex] + (int) (portion * podLength * Math.cos(ownAngle));
                coloredPodXCoords[3] = basePolygonXCoords[nextIndex];

                coloredPodYCoords[0] = basePolygonYCoords[i];
                coloredPodYCoords[1] = basePolygonYCoords[i] + (int) (portion * podLength * Math.sin(ownAngle));
                coloredPodYCoords[2] = basePolygonYCoords[nextIndex] + (int) (portion * podLength * Math.sin(ownAngle));
                coloredPodYCoords[3] = basePolygonYCoords[nextIndex];

                g.setColor(ithScenario.getColor());
                g.fillPolygon(coloredPodXCoords, coloredPodYCoords, 4);
                g.drawLine(basePolygonXCoords[i], basePolygonYCoords[i], basePolygonXCoords[nextIndex], basePolygonYCoords[nextIndex]);

                if (TradeoffsPanel.uncertainty == TradeoffsPanel.ERROR_BAR) {
                    int percent = Session.getErrorBarPercentage();
                    double[] errors = ithScenario.getErrorBar(indicator, percent);

                    double portion1 = (errors[0] - min)/(max - min);
                    double portion2 = (errors[1] - min)/(max - min);

                    int xCommon = (int) (0.5 * (basePolygonXCoords[i] + basePolygonXCoords[nextIndex]));
                    int xError1Line = xCommon + (int) (portion1 * podLength * Math.cos(ownAngle));
                    int xError2Line = xCommon + (int) (portion2 * podLength * Math.cos(ownAngle));

                    int yCommon = (int) (0.5 * (basePolygonYCoords[i] + basePolygonYCoords[nextIndex]));
                    int yError1Line = yCommon + (int) (portion1 * podLength * Math.sin(ownAngle));
                    int yError2Line = yCommon + (int) (portion2 * podLength * Math.sin(ownAngle));

                    g.setStroke(new BasicStroke(2.0f));
                    g.setColor(Constants.MAIN_COLOR);
                    g.drawLine(xError1Line, yError1Line, xError2Line, yError2Line);
                    g.setStroke(new BasicStroke(1.0f));
                }
            } else {
                java.util.List<Double> pdfValues = ithScenario.getPdfValues().get(indicator);
                double maxValue = ithScenario.getPdfMax().get(indicator);
                int binNum = pdfValues.size();
                double[] scenarioMinMax = ithScenario.getErrorBar(indicator, 100);
                for (int j = 0; j < binNum-1; ++j) {
                    int grayScale = (int)(255*(1-pdfValues.get(j)/maxValue));
                    Color color = new Color(grayScale, grayScale, grayScale);

                    double podMinValue = scenarioMinMax[0] + (j*1.0/binNum) * (scenarioMinMax[1]-scenarioMinMax[0]);
                    double podMaxValue = scenarioMinMax[0] + ((j+1)*1.0/binNum) * (scenarioMinMax[1]-scenarioMinMax[0]);

                    double portion1 = (podMinValue - min)/(max - min);
                    double portion2 = (podMaxValue - min)/(max - min);

                    int[] coloredPodXCoords = new int[4];
                    int[] coloredPodYCoords = new int[4];

                    coloredPodXCoords[0] = basePolygonXCoords[i] + (int) (portion1 * podLength * Math.cos(ownAngle));
                    coloredPodXCoords[1] = basePolygonXCoords[i] + (int) (portion2 * podLength * Math.cos(ownAngle));
                    coloredPodXCoords[2] = basePolygonXCoords[nextIndex] + (int) (portion2 * podLength * Math.cos(ownAngle));
                    coloredPodXCoords[3] = basePolygonXCoords[nextIndex] + (int) (portion1 * podLength * Math.cos(ownAngle));

                    coloredPodYCoords[0] = basePolygonYCoords[i] + (int) (portion1 * podLength * Math.sin(ownAngle));
                    coloredPodYCoords[1] = basePolygonYCoords[i] + (int) (portion2 * podLength * Math.sin(ownAngle));
                    coloredPodYCoords[2] = basePolygonYCoords[nextIndex] + (int) (portion2 * podLength * Math.sin(ownAngle));
                    coloredPodYCoords[3] = basePolygonYCoords[nextIndex] + (int) (portion1 * podLength * Math.sin(ownAngle));

                    g.setColor(color);
                    g.fillPolygon(coloredPodXCoords, coloredPodYCoords, 4);
                }
            }
        }
    }

    private void drawBorder(Graphics2D g) {
        if (TradeoffsPanel.sortByIndicator == null)
            return;

        if (indicator.equals(TradeoffsPanel.sortByIndicator)) {
            g.setColor(Constants.MAIN_COLOR);
            g.setStroke(new BasicStroke(2.0f));
            g.drawRect(x+(int) (0.5*(w-plotLength)), y+(int) (0.5*(h-plotLength)), plotLength, plotLength);
            g.setStroke(new BasicStroke(1.0f));
        }
    }

    public void showToolTip(int xMouse, int yMouse) {
        int stickNum = Session.getScenarios().size();
        double distanceToCentre = Math.sqrt((xMouse-xCentre)*(xMouse-xCentre) +
                                            (yMouse-yCentre)*(yMouse-yCentre));
        double mouseAngle = Util.calculateAngle(xMouse, yMouse, xCentre, yCentre);

        boolean toolTipSet = false;
        for (int i = 0; i < stickNum; ++i) {
            Scenario ithScenario = Session.getScenario(i);
            //double ownAngle = getAngle(ithScenario);
            double ownAngle = i*2*Math.PI / Session.getScenarios().size();

            if (distanceToCentre > podLength)
                break;

            if (ownAngle - angelDif <= mouseAngle && mouseAngle <= ownAngle + angelDif) {
                setToolTipText(ithScenario.getName());
                toolTipSet = true;
                break;
            }

        }

        if (!toolTipSet)
            setToolTipText("");
    }

    /*private double getAngle(Scenario scenario) {
        int scenarioNum = Session.getScenarios().size();

        int scenarioRank = Session.getScenarioRank(scenario);
        double ownAngle = scenarioRank * 2*Math.PI / scenarioNum;

        return ownAngle;
    }*/

}
